package javax.swing;

import java.io.InvalidObjectException;
import java.util.Iterator;
import java.util.Vector;
import java.io.ObjectInputValidation;
import java.awt.event.FocusEvent;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.Cursor;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleKeyBinding;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleRole;
import javax.accessibility.Accessible;
import javax.swing.border.CompoundBorder;
import javax.swing.border.TitledBorder;
import java.awt.event.FocusListener;
import javax.accessibility.AccessibleExtendedComponent;
import java.awt.event.ActionEvent;
import javax.accessibility.AccessibleContext;
import java.io.ObjectOutputStream;
import java.io.IOException;
import javax.swing.table.JTableHeader;
import java.util.ArrayList;
import sun.awt.SunToolkit;
import java.beans.PropertyChangeListener;
import java.util.EventListener;
import javax.swing.event.AncestorListener;
import java.beans.VetoableChangeListener;
import java.beans.PropertyVetoException;
import java.awt.peer.LightweightPeer;
import java.awt.AWTKeyStroke;
import javax.accessibility.AccessibleState;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.util.Locale;
import java.awt.Color;
import java.util.Enumeration;
import java.awt.event.ActionListener;
import javax.swing.border.AbstractBorder;
import java.awt.Insets;
import java.beans.Transient;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.FocusTraversalPolicy;
import sun.swing.UIClientPropertyKey;
import sun.swing.SwingUtilities2;
import java.applet.Applet;
import java.awt.Window;
import java.util.HashSet;
import java.awt.Graphics;
import sun.awt.RequestFocusController;
import java.awt.Rectangle;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.awt.Component;
import javax.swing.border.Border;
import java.beans.VetoableChangeSupport;
import javax.swing.event.EventListenerList;
import javax.swing.plaf.ComponentUI;
import java.util.Set;
import java.io.ObjectInputStream;
import java.util.Hashtable;
import java.io.Serializable;
import java.awt.Container;

public abstract class JComponent extends Container implements Serializable, TransferHandler.HasGetTransferHandler
{
    private static final String uiClassID = "ComponentUI";
    private static final Hashtable<ObjectInputStream, JComponent.ReadObjectCallback> readObjectCallbacks;
    private static Set<KeyStroke> managingFocusForwardTraversalKeys;
    private static Set<KeyStroke> managingFocusBackwardTraversalKeys;
    private static final int NOT_OBSCURED = 0;
    private static final int PARTIALLY_OBSCURED = 1;
    private static final int COMPLETELY_OBSCURED = 2;
    static boolean DEBUG_GRAPHICS_LOADED;
    private static final Object INPUT_VERIFIER_SOURCE_KEY;
    private boolean isAlignmentXSet;
    private float alignmentX;
    private boolean isAlignmentYSet;
    private float alignmentY;
    protected transient ComponentUI ui;
    protected EventListenerList listenerList;
    private transient ArrayTable clientProperties;
    private VetoableChangeSupport vetoableChangeSupport;
    private boolean autoscrolls;
    private Border border;
    private int flags;
    private InputVerifier inputVerifier;
    private boolean verifyInputWhenFocusTarget;
    transient Component paintingChild;
    public static final int WHEN_FOCUSED = 0;
    public static final int WHEN_ANCESTOR_OF_FOCUSED_COMPONENT = 1;
    public static final int WHEN_IN_FOCUSED_WINDOW = 2;
    public static final int UNDEFINED_CONDITION = -1;
    private static final String KEYBOARD_BINDINGS_KEY = "_KeyboardBindings";
    private static final String WHEN_IN_FOCUSED_WINDOW_BINDINGS = "_WhenInFocusedWindow";
    public static final String TOOL_TIP_TEXT_KEY = "ToolTipText";
    private static final String NEXT_FOCUS = "nextFocus";
    private JPopupMenu popupMenu;
    private static final int IS_DOUBLE_BUFFERED = 0;
    private static final int ANCESTOR_USING_BUFFER = 1;
    private static final int IS_PAINTING_TILE = 2;
    private static final int IS_OPAQUE = 3;
    private static final int KEY_EVENTS_ENABLED = 4;
    private static final int FOCUS_INPUTMAP_CREATED = 5;
    private static final int ANCESTOR_INPUTMAP_CREATED = 6;
    private static final int WIF_INPUTMAP_CREATED = 7;
    private static final int ACTIONMAP_CREATED = 8;
    private static final int CREATED_DOUBLE_BUFFER = 9;
    private static final int IS_PRINTING = 11;
    private static final int IS_PRINTING_ALL = 12;
    private static final int IS_REPAINTING = 13;
    private static final int WRITE_OBJ_COUNTER_FIRST = 14;
    private static final int RESERVED_1 = 15;
    private static final int RESERVED_2 = 16;
    private static final int RESERVED_3 = 17;
    private static final int RESERVED_4 = 18;
    private static final int RESERVED_5 = 19;
    private static final int RESERVED_6 = 20;
    private static final int WRITE_OBJ_COUNTER_LAST = 21;
    private static final int REQUEST_FOCUS_DISABLED = 22;
    private static final int INHERITS_POPUP_MENU = 23;
    private static final int OPAQUE_SET = 24;
    private static final int AUTOSCROLLS_SET = 25;
    private static final int FOCUS_TRAVERSAL_KEYS_FORWARD_SET = 26;
    private static final int FOCUS_TRAVERSAL_KEYS_BACKWARD_SET = 27;
    private transient AtomicBoolean revalidateRunnableScheduled;
    private static List<Rectangle> tempRectangles;
    private InputMap focusInputMap;
    private InputMap ancestorInputMap;
    private ComponentInputMap windowInputMap;
    private ActionMap actionMap;
    private static final String defaultLocale = "JComponent.defaultLocale";
    private static Component componentObtainingGraphicsFrom;
    private static Object componentObtainingGraphicsFromLock;
    private transient Object aaTextInfo;
    static final RequestFocusController focusController;
    
    static Graphics safelyGetGraphics(Component component) {
        return safelyGetGraphics(component, SwingUtilities.getRoot(component));
    }
    
    static Graphics safelyGetGraphics(Component component, Component componentObtainingGraphicsFrom) {
        synchronized (JComponent.componentObtainingGraphicsFromLock) {
            JComponent.componentObtainingGraphicsFrom = componentObtainingGraphicsFrom;
            Graphics graphics = component.getGraphics();
            JComponent.componentObtainingGraphicsFrom = null;
            return graphics;
        }
    }
    
    static void getGraphicsInvoked(Component component) {
        if (!isComponentObtainingGraphicsFrom(component)) {
            JRootPane rootPane = ((RootPaneContainer)component).getRootPane();
            if (rootPane != null) {
                rootPane.disableTrueDoubleBuffering();
            }
        }
    }
    
    private static boolean isComponentObtainingGraphicsFrom(Component component) {
        synchronized (JComponent.componentObtainingGraphicsFromLock) {
            return JComponent.componentObtainingGraphicsFrom == component;
        }
    }
    
    static Set<KeyStroke> getManagingFocusForwardTraversalKeys() {
        synchronized (JComponent.class) {
            if (JComponent.managingFocusForwardTraversalKeys == null) {
                (JComponent.managingFocusForwardTraversalKeys = new HashSet(1)).add(KeyStroke.getKeyStroke(9, 2));
            }
        }
        return JComponent.managingFocusForwardTraversalKeys;
    }
    
    static Set<KeyStroke> getManagingFocusBackwardTraversalKeys() {
        synchronized (JComponent.class) {
            if (JComponent.managingFocusBackwardTraversalKeys == null) {
                (JComponent.managingFocusBackwardTraversalKeys = new HashSet(1)).add(KeyStroke.getKeyStroke(9, 3));
            }
        }
        return JComponent.managingFocusBackwardTraversalKeys;
    }
    
    private static Rectangle fetchRectangle() {
        synchronized (JComponent.tempRectangles) {
            int size = JComponent.tempRectangles.size();
            Rectangle rectangle;
            if (size > 0) {
                rectangle = (Rectangle)JComponent.tempRectangles.remove(size - 1);
            }
            else {
                rectangle = new Rectangle(0, 0, 0, 0);
            }
            return rectangle;
        }
    }
    
    private static void recycleRectangle(Rectangle rectangle) {
        synchronized (JComponent.tempRectangles) {
            JComponent.tempRectangles.add(rectangle);
        }
    }
    
    public void setInheritsPopupMenu(boolean b) {
        int flag = this.getFlag(23);
        this.setFlag(23, b);
        this.firePropertyChange("inheritsPopupMenu", (boolean)flag, b);
    }
    
    public boolean getInheritsPopupMenu() {
        return this.getFlag(23);
    }
    
    public void setComponentPopupMenu(JPopupMenu popupMenu) {
        if (popupMenu != null) {
            this.enableEvents(16L);
        }
        this.firePropertyChange("componentPopupMenu", this.popupMenu, this.popupMenu = popupMenu);
    }
    
    public JPopupMenu getComponentPopupMenu() {
        if (!this.getInheritsPopupMenu()) {
            return this.popupMenu;
        }
        if (this.popupMenu == null) {
            Container container = this.getParent();
            while (container != null) {
                if ((container instanceof JComponent)) {
                    return ((JComponent)container).getComponentPopupMenu();
                }
                if (!(container instanceof Window)) {
                    if ((container instanceof Applet)) {
                        break;
                    }
                    container = container.getParent();
                }
                else {
                    break;
                }
            }
            return null;
        }
        return this.popupMenu;
    }
    
    public JComponent() {
        this.listenerList = new EventListenerList();
        this.inputVerifier = null;
        this.verifyInputWhenFocusTarget = true;
        this.revalidateRunnableScheduled = new AtomicBoolean(false);
        this.enableEvents(8L);
        if (this.isManagingFocus()) {
            LookAndFeel.installProperty(this, "focusTraversalKeysForward", getManagingFocusForwardTraversalKeys());
            LookAndFeel.installProperty(this, "focusTraversalKeysBackward", getManagingFocusBackwardTraversalKeys());
        }
        super.setLocale(getDefaultLocale());
    }
    
    public void updateUI() {
    }
    
    protected void setUI(ComponentUI componentUI) {
        this.uninstallUIAndProperties();
        this.aaTextInfo = UIManager.getDefaults().get(SwingUtilities2.AA_TEXT_PROPERTY_KEY);
        ComponentUI ui = this.ui;
        this.ui = componentUI;
        if (this.ui != null) {
            this.ui.installUI(this);
        }
        this.firePropertyChange("UI", ui, componentUI);
        this.revalidate();
        this.repaint();
    }
    
    private void uninstallUIAndProperties() {
        if (this.ui != null) {
            this.ui.uninstallUI(this);
            if (this.clientProperties != null) {
                synchronized (this.clientProperties) {
                    Object[] keys = this.clientProperties.getKeys((Object[])null);
                    if (keys != null) {
                        for (Object o : keys) {
                            if ((o instanceof UIClientPropertyKey)) {
                                this.putClientProperty(o, null);
                            }
                        }
                    }
                }
            }
        }
    }
    
    public String getUIClassID() {
        return "ComponentUI";
    }
    
    protected Graphics getComponentGraphics(Graphics graphics) {
        if (this.ui != null && JComponent.DEBUG_GRAPHICS_LOADED && DebugGraphics.debugComponentCount() != 0 && this.shouldDebugGraphics() != 0 && !(graphics instanceof DebugGraphics)) {
            graphics = new DebugGraphics(graphics, this);
        }
        graphics.setColor(this.getForeground());
        graphics.setFont(this.getFont());
        return graphics;
    }
    
    protected void paintComponent(Graphics graphics) {
        if (this.ui != null) {
            Graphics g = (graphics == null) ? null : graphics.create();
            try {
                this.ui.update(g, this);
            }
            finally {
                g.dispose();
            }
        }
    }
    
    protected void paintChildren(Graphics graphics) {
        synchronized (this.getTreeLock()) {
            int i = this.getComponentCount() - 1;
            if (i < 0) {
                return;
            }
            if (this.paintingChild != null && this.paintingChild instanceof JComponent && this.paintingChild.isOpaque()) {
                while (i >= 0) {
                    if (this.getComponent(i) == this.paintingChild) {
                        break;
                    }
                    --i;
                }
            }
            Rectangle fetchRectangle = fetchRectangle();
            int n = !this.isOptimizedDrawingEnabled() && this.checkIfChildObscuredBySibling();
            Rectangle clipBounds = null;
            if (!(n == 0)) {
                clipBounds = graphics.getClipBounds();
                if (clipBounds == null) {
                    clipBounds = new Rectangle(0, 0, this.getWidth(), this.getHeight());
                }
            }
            int flag = this.getFlag(11);
            Window windowAncestor = SwingUtilities.getWindowAncestor(this);
            int n2 = windowAncestor == null || windowAncestor.isOpaque();
            while (i >= 0) {
                Component component = this.getComponent(i);
                Label_0645: {
                    if (component != null) {
                        int n3 = component instanceof JComponent;
                        if ((n2 == 0 || n3 || isLightweightComponent(component)) && component.isVisible()) {
                            Rectangle bounds = component.getBounds(fetchRectangle);
                            if (graphics.hitClip(bounds.x, bounds.y, bounds.width, bounds.height)) {
                                if (n && i > 0) {
                                    int x = bounds.x;
                                    int y = bounds.y;
                                    int width = bounds.width;
                                    int height = bounds.height;
                                    SwingUtilities.computeIntersection(clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height, bounds);
                                    if (this.getObscuredState(i, bounds.x, bounds.y, bounds.width, bounds.height) == 2) {
                                        break Label_0645;
                                    }
                                    bounds.x = x;
                                    bounds.y = y;
                                    bounds.width = width;
                                    bounds.height = height;
                                }
                                Graphics create = graphics.create(bounds.x, bounds.y, bounds.width, bounds.height);
                                create.setColor(component.getForeground());
                                create.setFont(component.getFont());
                                boolean n4 = false;
                                try {
                                    if (!(n3 == 0)) {
                                        if (this.getFlag(1)) {
                                            ((JComponent)component).setFlag(1, true);
                                            n4 = true;
                                        }
                                        if (this.getFlag(2)) {
                                            ((JComponent)component).setFlag(2, true);
                                            n4 = true;
                                        }
                                        if (flag == 0) {
                                            component.paint(create);
                                        }
                                        else if (!this.getFlag(12)) {
                                            component.print(create);
                                        }
                                        else {
                                            component.printAll(create);
                                        }
                                    }
                                    else if (flag == 0) {
                                        component.paint(create);
                                    }
                                    else if (!this.getFlag(12)) {
                                        component.print(create);
                                    }
                                    else {
                                        component.printAll(create);
                                    }
                                }
                                finally {
                                    create.dispose();
                                    if (!(n4 == 0)) {
                                        ((JComponent)component).setFlag(1, false);
                                        ((JComponent)component).setFlag(2, false);
                                    }
                                }
                            }
                        }
                    }
                }
                --i;
            }
            recycleRectangle(fetchRectangle);
        }
    }
    
    protected void paintBorder(Graphics graphics) {
        Border border = this.getBorder();
        if (border != null) {
            border.paintBorder(this, graphics, 0, 0, this.getWidth(), this.getHeight());
        }
    }
    
    @Override
    public void update(Graphics graphics) {
        this.paint(graphics);
    }
    
    @Override
    public void paint(Graphics graphics) {
        boolean n = false;
        if (this.getWidth() <= 0 || this.getHeight() <= 0) {
            return;
        }
        Graphics create = this.getComponentGraphics(graphics).create();
        try {
            RepaintManager currentManager = RepaintManager.currentManager(this);
            Rectangle clipBounds = create.getClipBounds();
            int x;
            int y;
            int n2;
            int n3;
            if (clipBounds == null) {
                y = (x = 0);
                n2 = this.getWidth();
                n3 = this.getHeight();
            }
            else {
                x = clipBounds.x;
                y = clipBounds.y;
                n2 = clipBounds.width;
                n3 = clipBounds.height;
            }
            if (n2 > this.getWidth()) {
                n2 = this.getWidth();
            }
            if (n3 > this.getHeight()) {
                n3 = this.getHeight();
            }
            if (this.getParent() != null && !(this.getParent() instanceof JComponent)) {
                this.adjustPaintFlags();
                n = true;
            }
            int flag = this.getFlag(11);
            if (flag == 0 && currentManager.isDoubleBufferingEnabled() && !this.getFlag(1) && this.isDoubleBuffered() && (this.getFlag(13) || currentManager.isPainting())) {
                currentManager.beginPaint();
                try {
                    currentManager.paint(this, this, create, x, y, n2, n3);
                }
                finally {
                    currentManager.endPaint();
                }
            }
            else {
                if (clipBounds == null) {
                    create.setClip(x, y, n2, n3);
                }
                if (!this.rectangleIsObscured(x, y, n2, n3)) {
                    if (flag == 0) {
                        this.paintComponent(create);
                        this.paintBorder(create);
                    }
                    else {
                        this.printComponent(create);
                        this.printBorder(create);
                    }
                }
                if (flag == 0) {
                    this.paintChildren(create);
                }
                else {
                    this.printChildren(create);
                }
            }
        }
        finally {
            create.dispose();
            if (!(n == 0)) {
                this.setFlag(1, false);
                this.setFlag(2, false);
                this.setFlag(11, false);
                this.setFlag(12, false);
            }
        }
        return;
    }
    
    void paintForceDoubleBuffered(Graphics graphics) {
        RepaintManager currentManager = RepaintManager.currentManager(this);
        Rectangle clipBounds = graphics.getClipBounds();
        currentManager.beginPaint();
        this.setFlag(13, true);
        try {
            currentManager.paint(this, this, graphics, clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height);
        }
        finally {
            currentManager.endPaint();
            this.setFlag(13, false);
        }
    }
    
    boolean isPainting() {
        while (this != null) {
            if (this instanceof JComponent && ((JComponent)this).getFlag(1)) {
                return true;
            }
            this = this.getParent();
        }
        return false;
    }
    
    private void adjustPaintFlags() {
        Container container = this.getParent();
        while (container != null) {
            if ((container instanceof JComponent)) {
                JComponent component = (JComponent)container;
                if (component.getFlag(1)) {
                    this.setFlag(1, true);
                }
                if (component.getFlag(2)) {
                    this.setFlag(2, true);
                }
                if (component.getFlag(11)) {
                    this.setFlag(11, true);
                }
                if (component.getFlag(12)) {
                    this.setFlag(12, true);
                    break;
                }
                break;
            }
            else {
                container = container.getParent();
            }
        }
    }
    
    @Override
    public void printAll(Graphics graphics) {
        this.setFlag(12, true);
        try {
            this.print(graphics);
        }
        finally {
            this.setFlag(12, false);
        }
    }
    
    @Override
    public void print(Graphics graphics) {
        this.setFlag(11, true);
        this.firePropertyChange("paintingForPrint", false, true);
        try {
            this.paint(graphics);
        }
        finally {
            this.setFlag(11, false);
            this.firePropertyChange("paintingForPrint", true, false);
        }
    }
    
    protected void printComponent(Graphics graphics) {
        this.paintComponent(graphics);
    }
    
    protected void printChildren(Graphics graphics) {
        this.paintChildren(graphics);
    }
    
    protected void printBorder(Graphics graphics) {
        this.paintBorder(graphics);
    }
    
    public boolean isPaintingTile() {
        return this.getFlag(2);
    }
    
    public final boolean isPaintingForPrint() {
        return this.getFlag(11);
    }
    
    @Deprecated
    public boolean isManagingFocus() {
        return false;
    }
    
    private void registerNextFocusableComponent() {
        this.registerNextFocusableComponent(this.getNextFocusableComponent());
    }
    
    private void registerNextFocusableComponent(Component component) {
        if (component == null) {
            return;
        }
        Container container = this.isFocusCycleRoot() ? this : this.getFocusCycleRootAncestor();
        FocusTraversalPolicy focusTraversalPolicy = container.getFocusTraversalPolicy();
        if (!(focusTraversalPolicy instanceof LegacyGlueFocusTraversalPolicy)) {
            focusTraversalPolicy = new LegacyGlueFocusTraversalPolicy(focusTraversalPolicy);
            container.setFocusTraversalPolicy(focusTraversalPolicy);
        }
        ((LegacyGlueFocusTraversalPolicy)focusTraversalPolicy).setNextFocusableComponent(this, component);
        return;
    }
    
    private void deregisterNextFocusableComponent() {
        Component nextFocusableComponent = this.getNextFocusableComponent();
        if (nextFocusableComponent == null) {
            return;
        }
        Container container = this.isFocusCycleRoot() ? this : this.getFocusCycleRootAncestor();
        if (container == null) {
            return;
        }
        FocusTraversalPolicy focusTraversalPolicy = container.getFocusTraversalPolicy();
        if ((focusTraversalPolicy instanceof LegacyGlueFocusTraversalPolicy)) {
            ((LegacyGlueFocusTraversalPolicy)focusTraversalPolicy).unsetNextFocusableComponent(this, nextFocusableComponent);
        }
        return;
    }
    
    @Deprecated
    public void setNextFocusableComponent(Component component) {
        int displayable = this.isDisplayable();
        if (!(displayable == 0)) {
            this.deregisterNextFocusableComponent();
        }
        this.putClientProperty("nextFocus", component);
        if (!(displayable == 0)) {
            this.registerNextFocusableComponent(component);
        }
    }
    
    @Deprecated
    public Component getNextFocusableComponent() {
        return (Component)this.getClientProperty("nextFocus");
    }
    
    public void setRequestFocusEnabled(boolean b) {
        this.setFlag(22, !b);
    }
    
    public boolean isRequestFocusEnabled() {
        return !this.getFlag(22);
    }
    
    @Override
    public void requestFocus() {
        super.requestFocus();
    }
    
    public boolean requestFocus(boolean temporary) {
        return super.requestFocus(temporary);
    }
    
    @Override
    public boolean requestFocusInWindow() {
        return super.requestFocusInWindow();
    }
    
    @Override
    protected boolean requestFocusInWindow(boolean temporary) {
        return super.requestFocusInWindow(temporary);
    }
    
    public void grabFocus() {
        this.requestFocus();
    }
    
    public void setVerifyInputWhenFocusTarget(boolean verifyInputWhenFocusTarget) {
        this.firePropertyChange("verifyInputWhenFocusTarget", this.verifyInputWhenFocusTarget, this.verifyInputWhenFocusTarget = verifyInputWhenFocusTarget);
    }
    
    public boolean getVerifyInputWhenFocusTarget() {
        return this.verifyInputWhenFocusTarget;
    }
    
    @Override
    public FontMetrics getFontMetrics(Font font) {
        return SwingUtilities2.getFontMetrics(this, font);
    }
    
    @Override
    public void setPreferredSize(Dimension preferredSize) {
        super.setPreferredSize(preferredSize);
    }
    
    @Transient
    @Override
    public Dimension getPreferredSize() {
        if (this.isPreferredSizeSet()) {
            return super.getPreferredSize();
        }
        Dimension preferredSize = null;
        if (this.ui != null) {
            preferredSize = this.ui.getPreferredSize(this);
        }
        return (preferredSize != null) ? preferredSize : super.getPreferredSize();
    }
    
    @Override
    public void setMaximumSize(Dimension maximumSize) {
        super.setMaximumSize(maximumSize);
    }
    
    @Transient
    @Override
    public Dimension getMaximumSize() {
        if (this.isMaximumSizeSet()) {
            return super.getMaximumSize();
        }
        Dimension maximumSize = null;
        if (this.ui != null) {
            maximumSize = this.ui.getMaximumSize(this);
        }
        return (maximumSize != null) ? maximumSize : super.getMaximumSize();
    }
    
    @Override
    public void setMinimumSize(Dimension minimumSize) {
        super.setMinimumSize(minimumSize);
    }
    
    @Transient
    @Override
    public Dimension getMinimumSize() {
        if (this.isMinimumSizeSet()) {
            return super.getMinimumSize();
        }
        Dimension minimumSize = null;
        if (this.ui != null) {
            minimumSize = this.ui.getMinimumSize(this);
        }
        return (minimumSize != null) ? minimumSize : super.getMinimumSize();
    }
    
    @Override
    public boolean contains(int n, int n2) {
        return (this.ui != null) ? this.ui.contains(this, n, n2) : super.contains(n, n2);
    }
    
    public void setBorder(Border border) {
        Border border2 = this.border;
        this.firePropertyChange("border", border2, this.border = border);
        if (border != border2) {
            if (border == null || border2 == null || !border.getBorderInsets(this).equals(border2.getBorderInsets(this))) {
                this.revalidate();
            }
            this.repaint();
        }
    }
    
    public Border getBorder() {
        return this.border;
    }
    
    @Override
    public Insets getInsets() {
        if (this.border != null) {
            return this.border.getBorderInsets(this);
        }
        return super.getInsets();
    }
    
    public Insets getInsets(Insets insets) {
        if (insets == null) {
            insets = new Insets(0, 0, 0, 0);
        }
        if (this.border != null) {
            if ((this.border instanceof AbstractBorder)) {
                return ((AbstractBorder)this.border).getBorderInsets(this, insets);
            }
            return this.border.getBorderInsets(this);
        }
        else {
            Insets insets2 = insets;
            Insets insets3 = insets;
            Insets insets4 = insets;
            Insets insets5 = insets;
            int n = 0;
            insets5.bottom = n;
            insets4.right = n;
            insets3.top = n;
            insets2.left = n;
            return insets;
        }
    }
    
    @Override
    public float getAlignmentY() {
        if (this.isAlignmentYSet) {
            return this.alignmentY;
        }
        return super.getAlignmentY();
    }
    
    public void setAlignmentY(float n) {
        this.alignmentY = ((n > 1.0f) ? 1.0f : ((n < 0.0f) ? 0.0f : n));
        this.isAlignmentYSet = true;
    }
    
    @Override
    public float getAlignmentX() {
        if (this.isAlignmentXSet) {
            return this.alignmentX;
        }
        return super.getAlignmentX();
    }
    
    public void setAlignmentX(float n) {
        this.alignmentX = ((n > 1.0f) ? 1.0f : ((n < 0.0f) ? 0.0f : n));
        this.isAlignmentXSet = true;
    }
    
    public void setInputVerifier(InputVerifier newValue) {
        InputVerifier oldValue = (InputVerifier)this.getClientProperty(ClientPropertyKey.JComponent_INPUT_VERIFIER);
        this.putClientProperty(ClientPropertyKey.JComponent_INPUT_VERIFIER, newValue);
        this.firePropertyChange("inputVerifier", oldValue, newValue);
    }
    
    public InputVerifier getInputVerifier() {
        return (InputVerifier)this.getClientProperty(ClientPropertyKey.JComponent_INPUT_VERIFIER);
    }
    
    @Override
    public Graphics getGraphics() {
        if (JComponent.DEBUG_GRAPHICS_LOADED && this.shouldDebugGraphics() != 0) {
            return new DebugGraphics(super.getGraphics(), this);
        }
        return super.getGraphics();
    }
    
    public void setDebugGraphicsOptions(int n) {
        DebugGraphics.setDebugOptions(this, n);
    }
    
    public int getDebugGraphicsOptions() {
        return DebugGraphics.getDebugOptions(this);
    }
    
    int shouldDebugGraphics() {
        return DebugGraphics.shouldComponentDebug(this);
    }
    
    public void registerKeyboardAction(ActionListener actionListener, String s, KeyStroke keyStroke, int n) {
        InputMap inputMap = this.getInputMap(n, true);
        if (inputMap != null) {
            ActionMap actionMap = this.getActionMap(true);
            JComponent.ActionStandin actionStandin = new JComponent.ActionStandin(actionListener, s);
            inputMap.put(keyStroke, actionStandin);
            if (actionMap != null) {
                actionMap.put(actionStandin, actionStandin);
            }
        }
    }
    
    private void registerWithKeyboardManager(boolean b) {
        InputMap inputMap = this.getInputMap(2, false);
        Hashtable hashtable = (Object)this.getClientProperty("_WhenInFocusedWindow");
        KeyStroke[] allKeys;
        if (inputMap != null) {
            allKeys = inputMap.allKeys();
            if (allKeys != null) {
                for (int i = allKeys.length - 1; i >= 0; --i) {
                    if (!b || hashtable == null || hashtable.get(allKeys[i]) == null) {
                        this.registerWithKeyboardManager(allKeys[i]);
                    }
                    if (hashtable != null) {
                        hashtable.remove(allKeys[i]);
                    }
                }
            }
        }
        else {
            allKeys = null;
        }
        if (hashtable != null && hashtable.size() > 0) {
            Enumeration<Object> keys = hashtable.keys();
            while (keys.hasMoreElements()) {
                this.unregisterWithKeyboardManager((KeyStroke)keys.nextElement());
            }
            hashtable.clear();
        }
        if (allKeys != null && allKeys.length > 0) {
            if (hashtable == null) {
                hashtable = new Hashtable(allKeys.length);
                this.putClientProperty("_WhenInFocusedWindow", hashtable);
            }
            for (int j = allKeys.length - 1; j >= 0; --j) {
                hashtable.put(allKeys[j], allKeys[j]);
            }
        }
        else {
            this.putClientProperty("_WhenInFocusedWindow", null);
        }
    }
    
    private void unregisterWithKeyboardManager() {
        Hashtable<Object, Object> hashtable = (Hashtable)this.getClientProperty("_WhenInFocusedWindow");
        if (hashtable != null && hashtable.size() > 0) {
            Enumeration<Object> keys = hashtable.keys();
            while (keys.hasMoreElements()) {
                this.unregisterWithKeyboardManager((KeyStroke)keys.nextElement());
            }
        }
        this.putClientProperty("_WhenInFocusedWindow", null);
    }
    
    void componentInputMapChanged(ComponentInputMap componentInputMap) {
        InputMap inputMap;
        for (inputMap = this.getInputMap(2, false); inputMap != componentInputMap && inputMap != null; inputMap = inputMap.getParent()) {}
        if (inputMap != null) {
            this.registerWithKeyboardManager(false);
        }
    }
    
    private void registerWithKeyboardManager(KeyStroke keyStroke) {
        KeyboardManager.getCurrentManager().registerKeyStroke(keyStroke, this);
    }
    
    private void unregisterWithKeyboardManager(KeyStroke keyStroke) {
        KeyboardManager.getCurrentManager().unregisterKeyStroke(keyStroke, this);
    }
    
    public void registerKeyboardAction(ActionListener actionListener, KeyStroke keyStroke, int n) {
        this.registerKeyboardAction(actionListener, null, keyStroke, n);
    }
    
    public void unregisterKeyboardAction(KeyStroke keyStroke) {
        ActionMap actionMap = this.getActionMap(false);
        for (int i = 0; i < 3; ++i) {
            InputMap inputMap = this.getInputMap(i, false);
            if (inputMap != null) {
                Object value = inputMap.get(keyStroke);
                if (actionMap != null && value != null) {
                    actionMap.remove(value);
                }
                inputMap.remove(keyStroke);
            }
        }
    }
    
    public KeyStroke[] getRegisteredKeyStrokes() {
        int[] array = new int[3];
        KeyStroke[][] array2 = new KeyStroke[3][];
        for (int i = 0; i < 3; ++i) {
            InputMap inputMap = this.getInputMap(i, false);
            array2[i] = ((inputMap != null) ? inputMap.allKeys() : null);
            array[i] = ((array2[i] != null) ? array2[i].length : 0);
        }
        KeyStroke[] array3 = new KeyStroke[array[0] + array[1] + array[2]];
        int j = 0;
        int n = 0;
        while (j < 3) {
            if (array[j] > 0) {
                System.arraycopy(array2[j], 0, array3, n, array[j]);
                n += array[j];
            }
            ++j;
        }
        return array3;
    }
    
    public int getConditionForKeyStroke(KeyStroke keyStroke) {
        int i = 0;
        while (i < 3) {
            InputMap inputMap = this.getInputMap(i, false);
            if (inputMap != null && inputMap.get(keyStroke) != null) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public ActionListener getActionForKeyStroke(KeyStroke keyStroke) {
        ActionMap actionMap = this.getActionMap(false);
        if (actionMap == null) {
            return null;
        }
        for (int i = 0; i < 3; ++i) {
            InputMap inputMap = this.getInputMap(i, false);
            if (inputMap != null) {
                Object value = inputMap.get(keyStroke);
                if (value != null) {
                    Action value2 = actionMap.get(value);
                    if ((value2 instanceof JComponent.ActionStandin)) {
                        return ((JComponent.ActionStandin)value2).actionListener;
                    }
                    return value2;
                }
            }
        }
        return null;
    }
    
    public void resetKeyboardActions() {
        for (int i = 0; i < 3; ++i) {
            InputMap inputMap = this.getInputMap(i, false);
            if (inputMap != null) {
                inputMap.clear();
            }
        }
        ActionMap actionMap = this.getActionMap(false);
        if (actionMap != null) {
            actionMap.clear();
        }
    }
    
    public final void setInputMap(int n, InputMap inputMap) {
        switch (n) {
            case 2: {
                if (inputMap != null && !(inputMap instanceof ComponentInputMap)) {
                    throw new IllegalArgumentException("WHEN_IN_FOCUSED_WINDOW InputMaps must be of type ComponentInputMap");
                }
                this.windowInputMap = (ComponentInputMap)inputMap;
                this.setFlag(7, true);
                this.registerWithKeyboardManager(false);
                break;
            }
            case 1: {
                this.ancestorInputMap = inputMap;
                this.setFlag(6, true);
                break;
            }
            case 0: {
                this.focusInputMap = inputMap;
                this.setFlag(5, true);
                break;
            }
            default: {
                throw new IllegalArgumentException("condition must be one of JComponent.WHEN_IN_FOCUSED_WINDOW, JComponent.WHEN_FOCUSED or JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT");
            }
        }
    }
    
    public final InputMap getInputMap(int n) {
        return this.getInputMap(n, true);
    }
    
    public final InputMap getInputMap() {
        return this.getInputMap(0, true);
    }
    
    public final void setActionMap(ActionMap actionMap) {
        this.actionMap = actionMap;
        this.setFlag(8, true);
    }
    
    public final ActionMap getActionMap() {
        return this.getActionMap(true);
    }
    
    final InputMap getInputMap(int n, boolean b) {
        switch (n) {
            case 0: {
                if (this.getFlag(5)) {
                    return this.focusInputMap;
                }
                if (b) {
                    InputMap inputMap = new InputMap();
                    this.setInputMap(n, inputMap);
                    return inputMap;
                }
                break;
            }
            case 1: {
                if (this.getFlag(6)) {
                    return this.ancestorInputMap;
                }
                if (b) {
                    InputMap inputMap2 = new InputMap();
                    this.setInputMap(n, inputMap2);
                    return inputMap2;
                }
                break;
            }
            case 2: {
                if (this.getFlag(7)) {
                    return this.windowInputMap;
                }
                if (b) {
                    ComponentInputMap componentInputMap = new ComponentInputMap(this);
                    this.setInputMap(n, componentInputMap);
                    return componentInputMap;
                }
                break;
            }
            default: {
                throw new IllegalArgumentException("condition must be one of JComponent.WHEN_IN_FOCUSED_WINDOW, JComponent.WHEN_FOCUSED or JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT");
            }
        }
        return null;
    }
    
    final ActionMap getActionMap(boolean b) {
        if (this.getFlag(8)) {
            return this.actionMap;
        }
        if (b) {
            ActionMap actionMap = new ActionMap();
            this.setActionMap(actionMap);
            return actionMap;
        }
        return null;
    }
    
    @Override
    public int getBaseline(int n, int n2) {
        super.getBaseline(n, n2);
        if (this.ui != null) {
            return this.ui.getBaseline(this, n, n2);
        }
        return -1;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior() {
        if (this.ui != null) {
            return this.ui.getBaselineResizeBehavior(this);
        }
        return Component.BaselineResizeBehavior.OTHER;
    }
    
    @Deprecated
    public boolean requestDefaultFocus() {
        Container container = this.isFocusCycleRoot() ? this : this.getFocusCycleRootAncestor();
        if (container == null) {
            return false;
        }
        Component defaultComponent = container.getFocusTraversalPolicy().getDefaultComponent(container);
        if (defaultComponent != null) {
            defaultComponent.requestFocus();
            return true;
        }
        return false;
    }
    
    @Override
    public void setVisible(boolean visible) {
        if (visible != this.isVisible()) {
            super.setVisible(visible);
            if (visible) {
                Container parent = this.getParent();
                if (parent != null) {
                    Rectangle bounds = this.getBounds();
                    parent.repaint(bounds.x, bounds.y, bounds.width, bounds.height);
                }
                this.revalidate();
            }
        }
    }
    
    @Override
    public void setEnabled(boolean enabled) {
        int enabled2 = this.isEnabled();
        super.setEnabled(enabled);
        this.firePropertyChange("enabled", (boolean)enabled2, enabled);
        if ((enabled ? 1 : 0) != enabled2) {
            this.repaint();
        }
    }
    
    @Override
    public void setForeground(Color color) {
        Color foreground = this.getForeground();
        super.setForeground(color);
        if ((foreground == null) ? (color == null || color.equals(foreground)) : foreground.equals(color)) {
            return;
        }
        this.repaint();
    }
    
    @Override
    public void setBackground(Color color) {
        Color background = this.getBackground();
        super.setBackground(color);
        if ((background == null) ? (color == null || color.equals(background)) : background.equals(color)) {
            return;
        }
        this.repaint();
    }
    
    @Override
    public void setFont(Font font) {
        Font font2 = this.getFont();
        super.setFont(font);
        if (font != font2) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public static Locale getDefaultLocale() {
        Locale default1 = (Locale)SwingUtilities.appContextGet("JComponent.defaultLocale");
        if (default1 == null) {
            default1 = Locale.getDefault();
            setDefaultLocale(default1);
            return default1;
        }
        return default1;
    }
    
    public static void setDefaultLocale(Locale locale) {
        SwingUtilities.appContextPut("JComponent.defaultLocale", locale);
    }
    
    protected void processComponentKeyEvent(KeyEvent keyEvent) {
    }
    
    @Override
    protected void processKeyEvent(KeyEvent e) {
        super.processKeyEvent(e);
        if (!e.isConsumed()) {
            this.processComponentKeyEvent(e);
        }
        int shouldProcess = JComponent.KeyboardState.shouldProcess(e);
        if (e.isConsumed()) {
            return;
        }
        if (shouldProcess && this.processKeyBindings(e, e.getID() == 401)) {
            e.consume();
        }
        return;
    }
    
    protected boolean processKeyBinding(KeyStroke keyStroke, KeyEvent keyEvent, int n, boolean b) {
        InputMap inputMap = this.getInputMap(n, false);
        ActionMap actionMap = this.getActionMap(false);
        if (inputMap != null && actionMap != null && this.isEnabled()) {
            Object value = inputMap.get(keyStroke);
            Action action = (value == null) ? null : actionMap.get(value);
            if (action != null) {
                return SwingUtilities.notifyAction(action, keyStroke, keyEvent, this, keyEvent.getModifiers());
            }
        }
        return false;
    }
    
    boolean processKeyBindings(KeyEvent keyEvent, boolean b) {
        if (!SwingUtilities.isValidKeyEventForKeyBindings(keyEvent)) {
            return false;
        }
        KeyStroke keyStroke = null;
        KeyStroke keyStroke2;
        if (keyEvent.getID() == 400) {
            keyStroke2 = KeyStroke.getKeyStroke(keyEvent.getKeyChar());
        }
        else {
            keyStroke2 = KeyStroke.getKeyStroke(keyEvent.getKeyCode(), keyEvent.getModifiers(), !b);
            if (keyEvent.getKeyCode() != keyEvent.getExtendedKeyCode()) {
                keyStroke = KeyStroke.getKeyStroke(keyEvent.getExtendedKeyCode(), keyEvent.getModifiers(), !b);
            }
        }
        if (keyStroke != null && this.processKeyBinding(keyStroke, keyEvent, 0, b)) {
            return true;
        }
        if (this.processKeyBinding(keyStroke2, keyEvent, 0, b)) {
            return true;
        }
        while (this != null && !(this instanceof Window) && !(this instanceof Applet)) {
            if ((this instanceof JComponent)) {
                if (keyStroke != null && ((JComponent)this).processKeyBinding(keyStroke, keyEvent, 1, b)) {
                    return true;
                }
                if (((JComponent)this).processKeyBinding(keyStroke2, keyEvent, 1, b)) {
                    return true;
                }
            }
            if (this instanceof JInternalFrame && processKeyBindingsForAllComponents(keyEvent, this, b)) {
                return true;
            }
            this = this.getParent();
        }
        return this != null && processKeyBindingsForAllComponents(keyEvent, this, b);
    }
    
    static boolean processKeyBindingsForAllComponents(KeyEvent keyEvent, Container owner, boolean b) {
        while (!KeyboardManager.getCurrentManager().fireKeyboardAction(keyEvent, b, owner)) {
            if ((owner instanceof Popup.HeavyWeightWindow)) {
                owner = ((Window)owner).getOwner();
            }
            else {
                return false;
            }
        }
        return true;
    }
    
    public void setToolTipText(String s) {
        String toolTipText = this.getToolTipText();
        this.putClientProperty("ToolTipText", s);
        ToolTipManager sharedInstance = ToolTipManager.sharedInstance();
        if (s != null) {
            if (toolTipText == null) {
                sharedInstance.registerComponent(this);
            }
        }
        else {
            sharedInstance.unregisterComponent(this);
        }
    }
    
    public String getToolTipText() {
        return (String)this.getClientProperty("ToolTipText");
    }
    
    public String getToolTipText(MouseEvent mouseEvent) {
        return this.getToolTipText();
    }
    
    public Point getToolTipLocation(MouseEvent mouseEvent) {
        return null;
    }
    
    public Point getPopupLocation(MouseEvent mouseEvent) {
        return null;
    }
    
    public JToolTip createToolTip() {
        JToolTip toolTip = new JToolTip();
        toolTip.setComponent(this);
        return toolTip;
    }
    
    public void scrollRectToVisible(Rectangle rectangle) {
        int x = this.getX();
        int y = this.getY();
        Container container;
        for (container = this.getParent(); container != null && !(container instanceof JComponent) && !(container instanceof CellRendererPane); container = container.getParent()) {
            Rectangle bounds = container.getBounds();
            x += bounds.x;
            y += bounds.y;
        }
        if (container != null && !(container instanceof CellRendererPane)) {
            rectangle.x += x;
            rectangle.y += y;
            ((JComponent)container).scrollRectToVisible(rectangle);
            rectangle.x -= x;
            rectangle.y -= y;
        }
    }
    
    public void setAutoscrolls(boolean autoscrolls) {
        this.setFlag(25, true);
        if (this.autoscrolls != autoscrolls) {
            this.autoscrolls = autoscrolls;
            if (autoscrolls) {
                this.enableEvents(16L);
                this.enableEvents(32L);
            }
            else {
                Autoscroller.stop(this);
            }
        }
    }
    
    public boolean getAutoscrolls() {
        return this.autoscrolls;
    }
    
    public void setTransferHandler(TransferHandler newValue) {
        TransferHandler oldValue = (TransferHandler)this.getClientProperty(ClientPropertyKey.JComponent_TRANSFER_HANDLER);
        this.putClientProperty(ClientPropertyKey.JComponent_TRANSFER_HANDLER, newValue);
        SwingUtilities.installSwingDropTargetAsNecessary(this, newValue);
        this.firePropertyChange("transferHandler", oldValue, newValue);
    }
    
    @Override
    public TransferHandler getTransferHandler() {
        return (TransferHandler)this.getClientProperty(ClientPropertyKey.JComponent_TRANSFER_HANDLER);
    }
    
    TransferHandler.DropLocation dropLocationForPoint(Point point) {
        return null;
    }
    
    Object setDropLocation(TransferHandler.DropLocation dropLocation, Object o, boolean b) {
        return null;
    }
    
    void dndDone() {
    }
    
    @Override
    protected void processMouseEvent(MouseEvent e) {
        if (this.autoscrolls && e.getID() == 502) {
            Autoscroller.stop(this);
        }
        super.processMouseEvent(e);
    }
    
    @Override
    protected void processMouseMotionEvent(MouseEvent e) {
        int n = 1;
        if (this.autoscrolls && e.getID() == 506) {
            n = !Autoscroller.isRunning(this);
            Autoscroller.processMouseDragged(e);
        }
        if (n != 0) {
            super.processMouseMotionEvent(e);
        }
    }
    
    void superProcessMouseMotionEvent(MouseEvent e) {
        super.processMouseMotionEvent(e);
    }
    
    void setCreatedDoubleBuffer(boolean b) {
        this.setFlag(9, b);
    }
    
    boolean getCreatedDoubleBuffer() {
        return this.getFlag(9);
    }
    
    @Deprecated
    @Override
    public void enable() {
        if (!this.isEnabled()) {
            super.enable();
            if (this.accessibleContext != null) {
                this.accessibleContext.firePropertyChange("AccessibleState", null, AccessibleState.ENABLED);
            }
        }
    }
    
    @Deprecated
    @Override
    public void disable() {
        if (this.isEnabled()) {
            super.disable();
            if (this.accessibleContext != null) {
                this.accessibleContext.firePropertyChange("AccessibleState", AccessibleState.ENABLED, null);
            }
        }
    }
    
    private ArrayTable getClientProperties() {
        if (this.clientProperties == null) {
            this.clientProperties = new ArrayTable();
        }
        return this.clientProperties;
    }
    
    public final Object getClientProperty(Object o) {
        if (o == SwingUtilities2.AA_TEXT_PROPERTY_KEY) {
            return this.aaTextInfo;
        }
        if (o == SwingUtilities2.COMPONENT_UI_PROPERTY_KEY) {
            return this.ui;
        }
        if (this.clientProperties == null) {
            return null;
        }
        synchronized (this.clientProperties) {
            return this.clientProperties.get(o);
        }
    }
    
    public final void putClientProperty(Object o, Object o2) {
        if (o == SwingUtilities2.AA_TEXT_PROPERTY_KEY) {
            this.aaTextInfo = o2;
            return;
        }
        if (o2 == null && this.clientProperties == null) {
            return;
        }
        ArrayTable clientProperties = this.getClientProperties();
        Object value;
        synchronized (clientProperties) {
            value = clientProperties.get(o);
            if (o2 != null) {
                clientProperties.put(o, o2);
            }
            else if (value != null) {
                clientProperties.remove(o);
            }
            else {
                return;
            }
        }
        this.clientPropertyChanged(o, value, o2);
        this.firePropertyChange(o.toString(), value, o2);
        return;
    }
    
    void clientPropertyChanged(Object o, Object o2, Object o3) {
    }
    
    void setUIProperty(String str, Object o) {
        if (str == "opaque") {
            if (!this.getFlag(24)) {
                this.setOpaque((boolean)o);
                this.setFlag(24, false);
            }
        }
        else if (str == "autoscrolls") {
            if (!this.getFlag(25)) {
                this.setAutoscrolls((boolean)o);
                this.setFlag(25, false);
            }
        }
        else if (str == "focusTraversalKeysForward") {
            if (!this.getFlag(26)) {
                super.setFocusTraversalKeys(0, (Set)o);
            }
        }
        else if (str == "focusTraversalKeysBackward") {
            if (!this.getFlag(27)) {
                super.setFocusTraversalKeys(1, (Set)o);
            }
        }
        else {
            throw new IllegalArgumentException("property \"" + str + "\" cannot be set using this method");
        }
    }
    
    @Override
    public void setFocusTraversalKeys(int id, Set<? extends AWTKeyStroke> keystrokes) {
        if (id == 0) {
            this.setFlag(26, true);
        }
        else if (id == 1) {
            this.setFlag(27, true);
        }
        super.setFocusTraversalKeys(id, keystrokes);
    }
    
    public static boolean isLightweightComponent(Component component) {
        return component.getPeer() instanceof LightweightPeer;
    }
    
    @Deprecated
    @Override
    public void reshape(int x, int y, int width, int height) {
        super.reshape(x, y, width, height);
    }
    
    @Override
    public Rectangle getBounds(Rectangle rectangle) {
        if (rectangle == null) {
            return new Rectangle(this.getX(), this.getY(), this.getWidth(), this.getHeight());
        }
        rectangle.setBounds(this.getX(), this.getY(), this.getWidth(), this.getHeight());
        return rectangle;
    }
    
    @Override
    public Dimension getSize(Dimension dimension) {
        if (dimension == null) {
            return new Dimension(this.getWidth(), this.getHeight());
        }
        dimension.setSize(this.getWidth(), this.getHeight());
        return dimension;
    }
    
    @Override
    public Point getLocation(Point point) {
        if (point == null) {
            return new Point(this.getX(), this.getY());
        }
        point.setLocation(this.getX(), this.getY());
        return point;
    }
    
    @Override
    public int getX() {
        return super.getX();
    }
    
    @Override
    public int getY() {
        return super.getY();
    }
    
    @Override
    public int getWidth() {
        return super.getWidth();
    }
    
    @Override
    public int getHeight() {
        return super.getHeight();
    }
    
    @Override
    public boolean isOpaque() {
        return this.getFlag(3);
    }
    
    public void setOpaque(boolean b) {
        int flag = this.getFlag(3);
        this.setFlag(3, b);
        this.setFlag(24, true);
        this.firePropertyChange("opaque", (boolean)flag, b);
    }
    
    boolean rectangleIsObscured(int n, int n2, int n3, int n4) {
        int componentCount = this.getComponentCount();
        int i = 0;
        while (i < componentCount) {
            Component component = this.getComponent(i);
            int x = component.getX();
            int y = component.getY();
            int width = component.getWidth();
            int height = component.getHeight();
            if (n >= x && n + n3 <= x + width && n2 >= y && n2 + n4 <= y + height && component.isVisible()) {
                return component instanceof JComponent && component.isOpaque();
            }
            ++i;
        }
        return false;
    }
    
    static final void computeVisibleRect(Component component, Rectangle rectangle) {
        Container parent = component.getParent();
        Rectangle bounds = component.getBounds();
        if (parent == null || parent instanceof Window || parent instanceof Applet) {
            rectangle.setBounds(0, 0, bounds.width, bounds.height);
        }
        else {
            computeVisibleRect(parent, rectangle);
            rectangle.x -= bounds.x;
            rectangle.y -= bounds.y;
            SwingUtilities.computeIntersection(0, 0, bounds.width, bounds.height, rectangle);
        }
    }
    
    public void computeVisibleRect(Rectangle rectangle) {
        computeVisibleRect(this, rectangle);
    }
    
    public Rectangle getVisibleRect() {
        Rectangle rectangle = new Rectangle();
        this.computeVisibleRect(rectangle);
        return rectangle;
    }
    
    public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
        super.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    public void firePropertyChange(String propertyName, int oldValue, int newValue) {
        super.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    @Override
    public void firePropertyChange(String propertyName, char oldValue, char newValue) {
        super.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    protected void fireVetoableChange(String propertyName, Object oldValue, Object newValue) throws PropertyVetoException {
        if (this.vetoableChangeSupport == null) {
            return;
        }
        this.vetoableChangeSupport.fireVetoableChange(propertyName, oldValue, newValue);
        return;
    }
    
    public synchronized void addVetoableChangeListener(VetoableChangeListener listener) {
        if (this.vetoableChangeSupport == null) {
            this.vetoableChangeSupport = new VetoableChangeSupport(this);
        }
        this.vetoableChangeSupport.addVetoableChangeListener(listener);
    }
    
    public synchronized void removeVetoableChangeListener(VetoableChangeListener listener) {
        if (this.vetoableChangeSupport == null) {
            return;
        }
        this.vetoableChangeSupport.removeVetoableChangeListener(listener);
        return;
    }
    
    public synchronized VetoableChangeListener[] getVetoableChangeListeners() {
        if (this.vetoableChangeSupport == null) {
            return new VetoableChangeListener[0];
        }
        return this.vetoableChangeSupport.getVetoableChangeListeners();
    }
    
    public Container getTopLevelAncestor() {
        while (this != null) {
            if (this instanceof Window || this instanceof Applet) {
                return this;
            }
            this = this.getParent();
        }
        return null;
    }
    
    private AncestorNotifier getAncestorNotifier() {
        return (AncestorNotifier)this.getClientProperty(ClientPropertyKey.JComponent_ANCESTOR_NOTIFIER);
    }
    
    public void addAncestorListener(AncestorListener ancestorListener) {
        AncestorNotifier ancestorNotifier = this.getAncestorNotifier();
        if (ancestorNotifier == null) {
            ancestorNotifier = new AncestorNotifier(this);
            this.putClientProperty(ClientPropertyKey.JComponent_ANCESTOR_NOTIFIER, ancestorNotifier);
        }
        ancestorNotifier.addAncestorListener(ancestorListener);
    }
    
    public void removeAncestorListener(AncestorListener ancestorListener) {
        AncestorNotifier ancestorNotifier = this.getAncestorNotifier();
        if (ancestorNotifier == null) {
            return;
        }
        ancestorNotifier.removeAncestorListener(ancestorListener);
        if (ancestorNotifier.listenerList.getListenerList().length == 0) {
            ancestorNotifier.removeAllListeners();
            this.putClientProperty(ClientPropertyKey.JComponent_ANCESTOR_NOTIFIER, null);
        }
        return;
    }
    
    public AncestorListener[] getAncestorListeners() {
        AncestorNotifier ancestorNotifier = this.getAncestorNotifier();
        if (ancestorNotifier == null) {
            return new AncestorListener[0];
        }
        return ancestorNotifier.getAncestorListeners();
    }
    
    @Override
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        EventListener[] listeners;
        if (clazz == AncestorListener.class) {
            listeners = (EventListener[])this.getAncestorListeners();
        }
        else if (clazz == VetoableChangeListener.class) {
            listeners = (EventListener[])this.getVetoableChangeListeners();
        }
        else if (clazz == PropertyChangeListener.class) {
            listeners = (EventListener[])this.getPropertyChangeListeners();
        }
        else {
            listeners = this.listenerList.getListeners(clazz);
        }
        if (listeners.length == 0) {
            return super.getListeners(clazz);
        }
        return listeners;
    }
    
    @Override
    public void addNotify() {
        super.addNotify();
        this.firePropertyChange("ancestor", null, this.getParent());
        this.registerWithKeyboardManager(false);
        this.registerNextFocusableComponent();
    }
    
    @Override
    public void removeNotify() {
        super.removeNotify();
        this.firePropertyChange("ancestor", this.getParent(), null);
        this.unregisterWithKeyboardManager();
        this.deregisterNextFocusableComponent();
        if (this.getCreatedDoubleBuffer()) {
            RepaintManager.currentManager(this).resetDoubleBuffer();
            this.setCreatedDoubleBuffer(false);
        }
        if (this.autoscrolls) {
            Autoscroller.stop(this);
        }
    }
    
    @Override
    public void repaint(long n, int n2, int n3, int n4, int n5) {
        RepaintManager.currentManager(SunToolkit.targetToAppContext(this)).addDirtyRegion(this, n2, n3, n4, n5);
    }
    
    public void repaint(Rectangle rectangle) {
        this.repaint(0L, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    @Override
    public void revalidate() {
        if (this.getParent() == null) {
            return;
        }
        if (SunToolkit.isDispatchThreadForAppContext(this)) {
            this.invalidate();
            RepaintManager.currentManager(this).addInvalidComponent(this);
        }
        else {
            if (this.revalidateRunnableScheduled.getAndSet(true)) {
                return;
            }
            SunToolkit.executeOnEventHandlerThread(this, ()/*  */ -> {
                this.revalidateRunnableScheduled.set(false);
                this.revalidate();
                return;
            });
        }
        return;
    }
    
    @Override
    public boolean isValidateRoot() {
        return false;
    }
    
    public boolean isOptimizedDrawingEnabled() {
        return true;
    }
    
    protected boolean isPaintingOrigin() {
        return false;
    }
    
    public void paintImmediately(int n, int n2, int n3, int n4) {
        if (!this.isShowing()) {
            return;
        }
        JComponent paintingOrigin = SwingUtilities.getPaintingOrigin(this);
        if (paintingOrigin != null) {
            Rectangle convertRectangle = SwingUtilities.convertRectangle(this, new Rectangle(n, n2, n3, n4), paintingOrigin);
            paintingOrigin.paintImmediately(convertRectangle.x, convertRectangle.y, convertRectangle.width, convertRectangle.height);
            return;
        }
        while (!this.isOpaque()) {
            Container parent = this.getParent();
            if (parent != null) {
                n += this.getX();
                n2 += this.getY();
                this = parent;
                if (!(this instanceof JComponent)) {
                    break;
                }
                continue;
            }
            else {
                break;
            }
        }
        if ((this instanceof JComponent)) {
            ((JComponent)this)._paintImmediately(n, n2, n3, n4);
        }
        else {
            this.repaint(n, n2, n3, n4);
        }
        return;
    }
    
    public void paintImmediately(Rectangle rectangle) {
        this.paintImmediately(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    boolean alwaysOnTop() {
        return false;
    }
    
    void setPaintingChild(Component paintingChild) {
        this.paintingChild = paintingChild;
    }
    
    void _paintImmediately(int x, int y, int width, int height) {
        int n = 0;
        int n2 = 0;
        boolean n3 = false;
        JComponent component = null;
        RepaintManager currentManager = RepaintManager.currentManager(this);
        ArrayList list = new ArrayList(7);
        int i = -1;
        int n4 = 0;
        int n5 = 0;
        Rectangle fetchRectangle = fetchRectangle();
        fetchRectangle.x = x;
        fetchRectangle.y = y;
        fetchRectangle.width = width;
        fetchRectangle.height = height;
        int n6 = this.alwaysOnTop() && this.isOpaque();
        if (!(n6 == 0)) {
            SwingUtilities.computeIntersection(0, 0, this.getWidth(), this.getHeight(), fetchRectangle);
            if (fetchRectangle.width == 0) {
                recycleRectangle(fetchRectangle);
                return;
            }
        }
        Container container = null;
        while (this != null && !(this instanceof Window) && !(this instanceof Applet)) {
            JComponent component2 = (this instanceof JComponent) ? ((JComponent)this) : null;
            list.add(this);
            if (n6 == 0 && component2 != null && !component2.isOptimizedDrawingEnabled()) {
                int n7;
                if (this != this) {
                    if (component2.isPaintingOrigin()) {
                        n7 = true;
                    }
                    else {
                        Component[] components;
                        int n8;
                        for (components = this.getComponents(), n8 = 0; n8 < components.length && components[n8] != container; ++n8) {}
                        switch (component2.getObscuredState(n8, fetchRectangle.x, fetchRectangle.y, fetchRectangle.width, fetchRectangle.height)) {
                            case 0: {
                                n7 = false;
                                break;
                            }
                            case 2: {
                                recycleRectangle(fetchRectangle);
                                return;
                            }
                            default: {
                                n7 = true;
                                break;
                            }
                        }
                    }
                }
                else {
                    n7 = false;
                }
                if (!(n7 == 0)) {
                    this = component2;
                    i = n4;
                    n2 = (n = 0);
                    n3 = false;
                }
            }
            ++n4;
            if (currentManager.isDoubleBufferingEnabled() && component2 != null && component2.isDoubleBuffered()) {
                n3 = true;
                component = component2;
            }
            if (n6 == 0) {
                int x2 = this.getX();
                int y2 = this.getY();
                SwingUtilities.computeIntersection(n5, n5, this.getWidth(), this.getHeight(), fetchRectangle);
                fetchRectangle.x += x2;
                fetchRectangle.y += y2;
                n += x2;
                n2 += y2;
            }
            container = this;
            this = this.getParent();
        }
        if (this == null || this.getPeer() == null || fetchRectangle.width <= 0 || fetchRectangle.height <= 0) {
            recycleRectangle(fetchRectangle);
            return;
        }
        this.setFlag(13, true);
        fetchRectangle.x -= n;
        fetchRectangle.y -= n2;
        if (this != this) {
            for (; i > 0; --i) {
                Component component3 = (Component)list.get(i);
                if ((component3 instanceof JComponent)) {
                    ((JComponent)component3).setPaintingChild((Component)list.get(i - 1));
                }
            }
        }
        try {
            Graphics safelyGetGraphics;
            if ((safelyGetGraphics = safelyGetGraphics(this, this)) != null) {
                try {
                    if (!(n3 == 0)) {
                        RepaintManager currentManager2 = RepaintManager.currentManager(component);
                        currentManager2.beginPaint();
                        try {
                            currentManager2.paint(this, component, safelyGetGraphics, fetchRectangle.x, fetchRectangle.y, fetchRectangle.width, fetchRectangle.height);
                        }
                        finally {
                            currentManager2.endPaint();
                        }
                    }
                    else {
                        safelyGetGraphics.setClip(fetchRectangle.x, fetchRectangle.y, fetchRectangle.width, fetchRectangle.height);
                        this.paint(safelyGetGraphics);
                    }
                }
                finally {
                    safelyGetGraphics.dispose();
                }
            }
            if (this != this) {
                for (; i > 0; --i) {
                    Component component4 = (Component)list.get(i);
                    if ((component4 instanceof JComponent)) {
                        ((JComponent)component4).setPaintingChild(null);
                    }
                }
            }
            this.setFlag(13, false);
        }
        finally {
            if (this != this) {
                for (; i > 0; --i) {
                    Component component5 = (Component)list.get(i);
                    if ((component5 instanceof JComponent)) {
                        ((JComponent)component5).setPaintingChild(null);
                    }
                }
            }
            this.setFlag(13, false);
        }
        recycleRectangle(fetchRectangle);
        return;
    }
    
    void paintToOffscreen(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.setFlag(1, true);
            if (n2 + n4 < n6 || n + n3 < n5) {
                this.setFlag(2, true);
            }
            if (this.getFlag(13)) {
                this.paint(graphics);
            }
            else {
                if (!this.rectangleIsObscured(n, n2, n3, n4)) {
                    this.paintComponent(graphics);
                    this.paintBorder(graphics);
                }
                this.paintChildren(graphics);
            }
        }
        finally {
            this.setFlag(1, false);
            this.setFlag(2, false);
        }
    }
    
    private int getObscuredState(int n, int n2, int n3, int n4, int n5) {
        int n6 = 0;
        Rectangle fetchRectangle = fetchRectangle();
        for (int i = n - 1; i >= 0; --i) {
            Component component = this.getComponent(i);
            if (component.isVisible()) {
                int opaque;
                if ((component instanceof JComponent)) {
                    opaque = component.isOpaque();
                    if (opaque == 0 && n6 == 1) {
                        continue;
                    }
                }
                else {
                    opaque = true;
                }
                Rectangle bounds = component.getBounds(fetchRectangle);
                if (opaque && n2 >= bounds.x && n2 + n4 <= bounds.x + bounds.width && n3 >= bounds.y && n3 + n5 <= bounds.y + bounds.height) {
                    recycleRectangle(fetchRectangle);
                    return 2;
                }
                if (n6 == 0 && n2 + n4 > bounds.x && n3 + n5 > bounds.y && n2 < bounds.x + bounds.width && n3 < bounds.y + bounds.height) {
                    n6 = 1;
                }
            }
        }
        recycleRectangle(fetchRectangle);
        return n6;
    }
    
    boolean checkIfChildObscuredBySibling() {
        return true;
    }
    
    private void setFlag(int n, boolean b) {
        if (b) {
            this.flags |= 1 << n;
        }
        else {
            this.flags &= ~(1 << n);
        }
    }
    
    private boolean getFlag(int n) {
        int n2 = 1 << n;
        return (this.flags & n2) == n2;
    }
    
    static void setWriteObjCounter(JComponent component, byte b) {
        component.flags = ((component.flags & 0xFFC03FFF) | b << 14);
    }
    
    static byte getWriteObjCounter(JComponent component) {
        return (byte)(component.flags >> 14 & 0xFF);
    }
    
    public void setDoubleBuffered(boolean b) {
        this.setFlag(0, b);
    }
    
    @Override
    public boolean isDoubleBuffered() {
        return this.getFlag(0);
    }
    
    public JRootPane getRootPane() {
        return SwingUtilities.getRootPane(this);
    }
    
    void compWriteObjectNotify() {
        int writeObjCounter = getWriteObjCounter(this);
        setWriteObjCounter(this, (byte)(writeObjCounter + 1));
        if (writeObjCounter != 0) {
            return;
        }
        this.uninstallUIAndProperties();
        if (this.getToolTipText() != null || this instanceof JTableHeader) {
            ToolTipManager.sharedInstance().unregisterComponent(this);
        }
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        JComponent.ReadObjectCallback readObjectCallback = (JComponent.ReadObjectCallback)JComponent.readObjectCallbacks.get(objectInputStream);
        if (readObjectCallback == null) {
            try {
                JComponent.readObjectCallbacks.put(objectInputStream, readObjectCallback = new JComponent.ReadObjectCallback(objectInputStream));
            }
            catch (Exception ex) {
                throw new IOException(ex.toString());
            }
        }
        JComponent.ReadObjectCallback.access$700(readObjectCallback, this);
        int int1 = objectInputStream.readInt();
        if (int1 > 0) {
            this.clientProperties = new ArrayTable();
            for (int i = 0; i < int1; ++i) {
                this.clientProperties.put(objectInputStream.readObject(), objectInputStream.readObject());
            }
        }
        if (this.getToolTipText() != null) {
            ToolTipManager.sharedInstance().registerComponent(this);
        }
        setWriteObjCounter(this, (byte)0);
        this.revalidateRunnableScheduled = new AtomicBoolean(false);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ComponentUI")) {
            int n = (byte)(getWriteObjCounter(this) - 1);
            setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
        ArrayTable.writeArrayTable(objectOutputStream, this.clientProperties);
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",alignmentX=" + this.alignmentX + ",alignmentY=" + this.alignmentY + ",border=" + ((this.border == null) ? "" : ((this.border == this) ? "this" : this.border.toString())) + ",flags=" + this.flags + ",maximumSize=" + (this.isMaximumSizeSet() ? this.getMaximumSize().toString() : "") + ",minimumSize=" + (this.isMinimumSizeSet() ? this.getMinimumSize().toString() : "") + ",preferredSize=" + (this.isPreferredSizeSet() ? this.getPreferredSize().toString() : "");
    }
    
    @Deprecated
    @Override
    public void hide() {
        int showing = this.isShowing();
        super.hide();
        if (!(showing == 0)) {
            Container parent = this.getParent();
            if (parent != null) {
                Rectangle bounds = this.getBounds();
                parent.repaint(bounds.x, bounds.y, bounds.width, bounds.height);
            }
            this.revalidate();
        }
    }
    
    static {
        readObjectCallbacks = new Hashtable(1);
        INPUT_VERIFIER_SOURCE_KEY = new StringBuilder("InputVerifierSourceKey");
        JComponent.tempRectangles = new ArrayList(11);
        JComponent.componentObtainingGraphicsFromLock = new StringBuilder("componentObtainingGraphicsFrom");
        focusController = new RequestFocusController() {
            public boolean acceptRequestFocus(Component component, Component component2, boolean b, boolean b2, CausedFocusEvent.Cause cause) {
                if (component2 == null || !(component2 instanceof JComponent)) {
                    return true;
                }
                if (component == null || !(component instanceof JComponent)) {
                    return true;
                }
                if (!((JComponent)component2).getVerifyInputWhenFocusTarget()) {
                    return true;
                }
                JComponent component3 = (JComponent)component;
                InputVerifier inputVerifier = component3.getInputVerifier();
                if (inputVerifier == null) {
                    return true;
                }
                Object appContextGet = SwingUtilities.appContextGet(JComponent.INPUT_VERIFIER_SOURCE_KEY);
                if (appContextGet == component3) {
                    return true;
                }
                SwingUtilities.appContextPut(JComponent.INPUT_VERIFIER_SOURCE_KEY, component3);
                try {
                    return inputVerifier.shouldYieldFocus(component3);
                }
                finally {
                    if (appContextGet != null) {
                        SwingUtilities.appContextPut(JComponent.INPUT_VERIFIER_SOURCE_KEY, appContextGet);
                    }
                    else {
                        SwingUtilities.appContextRemove(JComponent.INPUT_VERIFIER_SOURCE_KEY);
                    }
                }
            }
        };
    }
    
    final class ActionStandin implements Action
    {
        private final ActionListener actionListener;
        private final String command;
        private final Action action;
        
        ActionStandin(ActionListener actionListener, String command) {
            this.actionListener = actionListener;
            if ((actionListener instanceof Action)) {
                this.action = (Action)actionListener;
            }
            else {
                this.action = null;
            }
            this.command = command;
        }
        
        @Override
        public Object getValue(String s) {
            if (s != null) {
                if (s.equals("ActionCommandKey")) {
                    return this.command;
                }
                if (this.action != null) {
                    return this.action.getValue(s);
                }
                if (s.equals("Name")) {
                    return "ActionStandin";
                }
            }
            return null;
        }
        
        @Override
        public boolean isEnabled() {
            return this.actionListener != null && (this.action == null || this.action.isEnabled());
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            if (this.actionListener != null) {
                this.actionListener.actionPerformed(actionEvent);
            }
        }
        
        @Override
        public void putValue(String s, Object o) {
        }
        
        @Override
        public void setEnabled(boolean b) {
        }
        
        @Override
        public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        @Override
        public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
    }
    
    static final class IntVector
    {
        int[] array;
        int count;
        int capacity;
        
        IntVector() {
            this.array = null;
            this.count = 0;
            this.capacity = 0;
        }
        
        int size() {
            return this.count;
        }
        
        int elementAt(int n) {
            return this.array[n];
        }
        
        void addElement(int n) {
            if (this.count == this.capacity) {
                this.capacity = (this.capacity + 2) * 2;
                int[] array = new int[this.capacity];
                if (this.count > 0) {
                    System.arraycopy(this.array, 0, array, 0, this.count);
                }
                this.array = array;
            }
            this.array[this.count++] = n;
        }
        
        void setElementAt(int n, int n2) {
            this.array[n2] = n;
        }
    }
    
    static class KeyboardState implements Serializable
    {
        private static final Object keyCodesKey;
        
        static JComponent.IntVector getKeyCodeArray() {
            JComponent.IntVector intVector = (JComponent.IntVector)SwingUtilities.appContextGet(JComponent.KeyboardState.keyCodesKey);
            if (intVector == null) {
                intVector = new JComponent.IntVector();
                SwingUtilities.appContextPut(JComponent.KeyboardState.keyCodesKey, intVector);
                return intVector;
            }
            return intVector;
        }
        
        static void registerKeyPressed(int n) {
            JComponent.IntVector keyCodeArray = getKeyCodeArray();
            int size = keyCodeArray.size();
            int i = 0;
            while (i < size) {
                if (keyCodeArray.elementAt(i) == -1) {
                    keyCodeArray.setElementAt(n, i);
                    return;
                }
                ++i;
            }
            keyCodeArray.addElement(n);
        }
        
        static void registerKeyReleased(int n) {
            JComponent.IntVector keyCodeArray = getKeyCodeArray();
            int size = keyCodeArray.size();
            int i = 0;
            while (i < size) {
                if (keyCodeArray.elementAt(i) == n) {
                    keyCodeArray.setElementAt(-1, i);
                    return;
                }
                ++i;
            }
        }
        
        static boolean keyIsPressed(int n) {
            JComponent.IntVector keyCodeArray = getKeyCodeArray();
            int size = keyCodeArray.size();
            int i = 0;
            while (i < size) {
                if (keyCodeArray.elementAt(i) == n) {
                    return true;
                }
                ++i;
            }
            return false;
        }
        
        static boolean shouldProcess(KeyEvent keyEvent) {
            switch (keyEvent.getID()) {
                case 401: {
                    if (!keyIsPressed(keyEvent.getKeyCode())) {
                        registerKeyPressed(keyEvent.getKeyCode());
                    }
                    return true;
                }
                case 402: {
                    if (keyIsPressed(keyEvent.getKeyCode()) || keyEvent.getKeyCode() == 154) {
                        registerKeyReleased(keyEvent.getKeyCode());
                        return true;
                    }
                    return false;
                }
                case 400: {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
        
        static {
            keyCodesKey = JComponent.KeyboardState.class;
        }
    }
    
    public abstract class AccessibleJComponent extends Container.AccessibleAWTContainer implements AccessibleExtendedComponent
    {
        private transient volatile int propertyListenersCount;
        @Deprecated
        protected FocusListener accessibleFocusHandler;
        
        protected AccessibleJComponent() {
            this.propertyListenersCount = 0;
            this.accessibleFocusHandler = null;
        }
        
        @Override
        public void addPropertyChangeListener(PropertyChangeListener listener) {
            super.addPropertyChangeListener(listener);
        }
        
        @Override
        public void removePropertyChangeListener(PropertyChangeListener listener) {
            super.removePropertyChangeListener(listener);
        }
        
        protected String getBorderTitle(Border border) {
            if ((border instanceof TitledBorder)) {
                return ((TitledBorder)border).getTitle();
            }
            if ((border instanceof CompoundBorder)) {
                String s = this.getBorderTitle(((CompoundBorder)border).getInsideBorder());
                if (s == null) {
                    s = this.getBorderTitle(((CompoundBorder)border).getOutsideBorder());
                    return s;
                }
                return s;
            }
            else {
                return null;
            }
        }
        
        @Override
        public String getAccessibleName() {
            String s = this.accessibleName;
            if (s == null) {
                s = (String)JComponent.this.getClientProperty("AccessibleName");
            }
            if (s == null) {
                s = this.getBorderTitle(JComponent.this.getBorder());
            }
            if (s == null) {
                Object clientProperty = JComponent.this.getClientProperty("labeledBy");
                if ((clientProperty instanceof Accessible)) {
                    AccessibleContext accessibleContext = ((Accessible)clientProperty).getAccessibleContext();
                    if (accessibleContext != null) {
                        s = accessibleContext.getAccessibleName();
                    }
                }
            }
            return s;
        }
        
        @Override
        public String getAccessibleDescription() {
            String s = this.accessibleDescription;
            if (s == null) {
                s = (String)JComponent.this.getClientProperty("AccessibleDescription");
            }
            if (s == null) {
                try {
                    s = this.getToolTipText();
                }
                catch (Exception ex) {}
            }
            if (s == null) {
                Object clientProperty = JComponent.this.getClientProperty("labeledBy");
                if ((clientProperty instanceof Accessible)) {
                    AccessibleContext accessibleContext = ((Accessible)clientProperty).getAccessibleContext();
                    if (accessibleContext != null) {
                        s = accessibleContext.getAccessibleDescription();
                    }
                }
            }
            return s;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.SWING_COMPONENT;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (JComponent.this.isOpaque()) {
                accessibleStateSet.add(AccessibleState.OPAQUE);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return super.getAccessibleChildrenCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int i) {
            return super.getAccessibleChild(i);
        }
        
        AccessibleExtendedComponent getAccessibleExtendedComponent() {
            return this;
        }
        
        @Override
        public String getToolTipText() {
            return JComponent.this.getToolTipText();
        }
        
        @Override
        public String getTitledBorderText() {
            Border border = JComponent.this.getBorder();
            if ((border instanceof TitledBorder)) {
                return ((TitledBorder)border).getTitle();
            }
            return null;
        }
        
        @Override
        public AccessibleKeyBinding getAccessibleKeyBinding() {
            Object clientProperty = JComponent.this.getClientProperty("labeledBy");
            if ((clientProperty instanceof Accessible)) {
                AccessibleContext accessibleContext = ((Accessible)clientProperty).getAccessibleContext();
                if (accessibleContext != null) {
                    AccessibleComponent accessibleComponent = accessibleContext.getAccessibleComponent();
                    if (!(accessibleComponent instanceof AccessibleExtendedComponent)) {
                        return null;
                    }
                    return ((AccessibleExtendedComponent)accessibleComponent).getAccessibleKeyBinding();
                }
            }
            return null;
        }
        
        protected class AccessibleContainerHandler implements ContainerListener
        {
            @Override
            public void componentAdded(ContainerEvent containerEvent) {
                Component child = containerEvent.getChild();
                if (child != null && child instanceof Accessible) {
                    JComponent.AccessibleJComponent.this.firePropertyChange("AccessibleChild", null, child.getAccessibleContext());
                }
            }
            
            @Override
            public void componentRemoved(ContainerEvent containerEvent) {
                Component child = containerEvent.getChild();
                if (child != null && child instanceof Accessible) {
                    JComponent.AccessibleJComponent.this.firePropertyChange("AccessibleChild", child.getAccessibleContext(), null);
                }
            }
        }
        
        protected class AccessibleFocusHandler implements FocusListener
        {
            @Override
            public void focusGained(FocusEvent focusEvent) {
                if (JComponent.this.accessibleContext != null) {
                    JComponent.this.accessibleContext.firePropertyChange("AccessibleState", null, AccessibleState.FOCUSED);
                }
            }
            
            @Override
            public void focusLost(FocusEvent focusEvent) {
                if (JComponent.this.accessibleContext != null) {
                    JComponent.this.accessibleContext.firePropertyChange("AccessibleState", AccessibleState.FOCUSED, null);
                }
            }
        }
    }
    
    private class ReadObjectCallback implements ObjectInputValidation
    {
        private final Vector<JComponent> roots;
        private final ObjectInputStream inputStream;
        
        ReadObjectCallback(ObjectInputStream inputStream) throws Exception {
            this.roots = new Vector(1);
            (this.inputStream = inputStream).registerValidation(this, 0);
        }
        
        @Override
        public void validateObject() throws InvalidObjectException {
            try {
                Iterator<JComponent> iterator = this.roots.iterator();
                while (iterator.hasNext()) {
                    SwingUtilities.updateComponentTreeUI((JComponent)iterator.next());
                }
            }
            finally {
                JComponent.readObjectCallbacks.remove(this.inputStream);
            }
        }
        
        private void registerComponent(JComponent parent) {
            for (JComponent component : this.roots) {
                while (parent != null) {
                    if (parent == component) {
                        return;
                    }
                    parent = parent.getParent();
                }
            }
            for (int i = 0; i < this.roots.size(); ++i) {
                Container container = ((JComponent)this.roots.elementAt(i)).getParent();
                while (container != null) {
                    if (container == parent) {
                        this.roots.removeElementAt(i--);
                        break;
                    }
                    container = container.getParent();
                }
            }
            this.roots.addElement(parent);
        }
        
        static void access$700(JComponent.ReadObjectCallback readObjectCallback, JComponent component) {
            readObjectCallback.registerComponent(component);
        }
    }
}
